function make_header
;+
; NAME:
;	make_header
; PURPOSE:
;	Creates an empty header with space for 50 keywords
; CALLING SEQUENCE:
;	<header> = make_header()
; INPUTS:
;	None
; OUTPUTS:
;	<header>: New header structure
; COMMON BLOCKS:
;	None
; RESTRICTIONS:
;	Used by IMSYS procedures
; MODIFICATION HISTORY:
;	BBT, July, 1991
;-
on_error, 2
key = {FITS_KEY, keyword: '', value: '', comment: ''}
return, replicate(key, 50)
end

function make_image, n_images
;+
; NAME:
;	make_image
; PURPOSE:
;	Creates a list of empty image headers 
; CALLING SEQUENCE:
;	<image_list> = make_image(n_images)
; INPUTS:
;	n_images: Number of image headers
; OUTPUTS:
;	<image_list>: New list of image headers
; COMMON BLOCKS:
;	None
; RESTRICTIONS:
;	Used by IMSYS procedures
; MODIFICATION HISTORY:
;	BBT, July, 1991
;-
on_error, 2
if (n_elements(n_images) eq 0) then begin
	message, 'Usage: image_list = make_image(n_images)'
endif
image = {FITS_HEADER, name: '', update: 0, header: make_header()}
return, replicate(image, n_images)
end

pro prolog
;+
; NAME:
;	prolog
; PURPOSE:
;	Creates an image list for all header files in header directory 
; CALLING SEQUENCE:
;	prolog
; INPUTS:
;	No explicit input
; OUTPUTS:
;	No explicit output
; COMMON BLOCKS:
;	DIR_ENV: Header and image directories, hdr_dir and img_dir
;	IMG_LST: List of image headers
; RESTRICTIONS:
;	Used by IMSYS procedures
; MODIFICATION HISTORY:
;	BBT, December, 1992
;-
on_error, 2
common DIR_ENV, hdr_dir, img_dir
common IMG_LST, ilist
hdr_dir	= getenv('HDRPATH')
img_dir = getenv('IMGPATH')
cd, hdr_dir, current=old_dir
hdr_lst	= findfile('*.HDR')
cd, old_dir
n_img	= n_elements(hdr_lst)
if (n_img gt 0) then begin
	ilist	= make_image(n_img)
	for i = 0, n_img-1 do begin
		hdr = hdr_lst(i)
		ilist(i).name = strlowcase(strmid(hdr, 0, strpos(hdr, '.')))
	endfor
endif
return
end

function rdheader, image
;+
; NAME:
;	rdheader
; PURPOSE:
;	Read keywords into image header 
; CALLING SEQUENCE:
;	<image> = rdheader(image)
; INPUTS:
;	image: Named, but empty image header
; OUTPUTS:
;	<image>: Same header containing keywords from header file
; COMMON BLOCKS:
;	DIR_ENV: Header and image directories, hdr_dir, img_dir
; RESTRICTIONS:
;	Used by IMSYS procedures
; MODIFICATION HISTORY:
;	BBT, December, 1992
;-
on_error, 2
on_ioerror, io_err
common DIR_ENV, hdr_dir, img_dir
if (n_elements(image) eq 0) then begin
	message, 'Usage: image = rdheader(image)'
endif
line = ''
cd, hdr_dir, current=old_dir
get_lun, unit
openr, unit, image.name + '.HDR', error=err
if (err ne 0) then goto, finish
for k = 0, n_elements(image.header) - 1 do begin
	readf, unit, line
	on_ioerror, finish
	s = strcompress(strtrim(line, 2))
	l = strpos(s, ' ')
	if (l eq -1) then l = strlen(s)
	image.header(k).keyword = strmid(s, 0, l)
	m = strpos(s, '\', l)
	if (m eq -1) then m = strlen(s)
	image.header(k).value = strtrim(strmid(s, l, m-l), 2)
	n = strlen(s)
	if (m lt n) then begin
		image.header(k).comment = strtrim(strmid(s, m+1, n-m-1), 2)
	endif
endfor
goto, finish
io_err:
message, !err_string, /noname, /ioerror
finish:
on_ioerror, null
free_lun, unit
cd, old_dir
return, image
end

function wrheader, image
;+
; NAME:
;	wdheader
; PURPOSE:
;	Writes keywords onto image header 
; CALLING SEQUENCE:
;	<image> = wdheader(image)
; INPUTS:
;	image: image header containing keywords
; OUTPUTS:
;	<image>: Updated version of same image header
; COMMON BLOCKS:
;	DIR_ENV: Header and image directories, hdr_dir, img_dir
; RESTRICTIONS:
;	Used by IMSYS procedures
; MODIFICATION HISTORY:
;	BBT, July, 1991
;-
on_error, 2
on_ioerror, io_err
common DIR_ENV, hdr_dir, img_dir
if (n_elements(image) eq 0) then begin
	message, 'Usage: image = wrheader(image)'
endif
cd, hdr_dir, current=old_dir
get_lun, unit
k = where(image.header.keyword ne '', count)
if (count gt 0) then begin
	openw, unit, image.name + '.HDR'
	for l = 0, count-1 do begin
		key = image.header(k(l)).keyword
		val = image.header(k(l)).value
		com = image.header(k(l)).comment
		if (com eq '') then begin
			printf, unit, key, '	', val
		endif else begin
			printf, unit, key, '	', val, '	\ ', com
		endelse
	endfor
	image.update = 0
endif else begin
	openw, unit, image.name + '.HDR', /delete
	close, unit
	image.update = 0
	image.name = ''
endelse
goto, finish
io_err:
on_ioerror, null
message, !err_string, /noname, /ioerror
finish:
free_lun, unit
cd, old_dir
return, image
end

pro epilog
;+
; NAME:
;	epilog
; PURPOSE:
;	Updates disk headers from current image list 
; CALLING SEQUENCE:
;	epilog
; INPUTS:
;	No explicit input
; OUTPUTS:
;	No explicit output
; COMMON BLOCKS:
;	IMG_LST: List of image headers
; RESTRICTIONS:
;	Used by IMSYS procedures
; MODIFICATION HISTORY:
;	BBT, July, 1991
;-
on_error, 2
common IMG_LST, ilist
if (n_elements(ilist) gt 0) then begin
	u = where(ilist.update, count)
	for i = 0, count-1 do ilist(u(i)) = wrheader(ilist(u(i)))
endif
return
end

pro list, search, names
;+
; NAME:
;	list
; PURPOSE:
;	Creates a list of image names according to a search string 
; CALLING SEQUENCE:
;	list[, search[, names]]
; INPUTS:
;	search:	Search string containing wild card (*)
; OUTPUTS:
;	names:	List of image names
; COMMON BLOCKS:
;	None
; RESTRICTIONS:
;	None
; MODIFICATION HISTORY:
;	BBT, December, 1992
;-
on_error, 2
if (n_params() eq 0) then search = '*'
cd, getenv('HDRPATH'), current = old_dir
files = findfile(search + '.HDR')
cd, old_dir
for i = 0, n_elements(files)-1 do begin
	file = files(i)
	files(i) = strlowcase(strmid(file, 0, strpos(file, '.')))
endfor
if (n_params() gt 1) then begin
	names = files
endif else begin
	for i = 0, n_elements(files)-1 do begin
		if (i gt 0 and i mod 8 eq 0) then print, ''
		print, format='(a9, $)', files(i) + '        '
	endfor
	print, ''
endelse
return
end

function getkey, name, keyword
;+
; NAME:
;	getkey
; PURPOSE:
;	Get the value of a keyword for a given image 
; CALLING SEQUENCE:
;	<string> = getkey(name, keyword)
; INPUTS:
;	name:    Image name
;	keyword: Keyword name
; OUTPUTS:
;	<string>: Value of given keyword
; COMMON BLOCKS:
;	IMG_LST: List of image headers
; RESTRICTIONS:
;	Used by IMSYS procedures
; MODIFICATION HISTORY:
;	BBT, July, 1991
;-
on_error, 2
common IMG_LST, ilist
if (n_elements(name) eq 0) then begin
	message, 'Usage: str = getkey(name, keyword)'
endif
if (n_elements(keyword) eq 0) then begin
	message, 'Usage: str = getkey(name, keyword)'
endif
if (n_elements(ilist) eq 0) then prolog
if (n_elements(ilist) eq 0) then begin
	message, 'Empty image list'
endif
i = where(ilist.name eq name, count)
if (count gt 1) then begin
	message, 'More than one image: ' + name
endif
if (count eq 0) then begin
	message, 'Image not found: ' + name
endif
k = where(ilist(i).header.keyword ne '', count)
if (count eq 0) then ilist(i) = rdheader(ilist(i))
k = where(ilist(i).header.keyword eq keyword, count)
if (count gt 1) then begin
	message, 'More than one keyword: ' + keyword
endif
if (count eq 0) then return, ''
return, ilist(i).header(k).value
end

function gethdr, name
;+
; NAME:
;	gethdr
; PURPOSE:
;	Get the header for a given image
; CALLING SEQUENCE:
;	<header> = gethdr(name)
; INPUTS:
;	name: Image name
; OUTPUTS:
;	<header>: Header containing all keywords for given image
; COMMON BLOCKS:
;	IMG_LST: List of image headers
; RESTRICTIONS:
;	Used by IMSYS procedures
; MODIFICATION HISTORY:
;	BBT, July, 1991
;-
on_error, 2
common IMG_LST, ilist
if (n_elements(name) eq 0) then begin
	message, 'Usage: header = gethdr(name)'
endif
if (n_elements(ilist) eq 0) then prolog
if (n_elements(ilist) eq 0) then begin
	message, 'Empty image list'
endif
i = where(ilist.name eq name, count)
if (count gt 1) then begin
	message, 'More than one image: ' + name
endif
if (count eq 0) then begin
	message, 'Image not found: ' + name
endif
k = where(ilist(i).header.keyword ne '', count)
if (count eq 0) then ilist(i) = rdheader(ilist(i))
k = where(ilist(i).header.keyword ne '', count)
if (count eq 0) then begin
	message, 'No keywords in header: ' + name
endif
return, ilist(i).header(k)
end

pro delhdr, name
;+
; NAME:
;	delhdr
; PURPOSE:
;	Delete the header for a given image
; CALLING SEQUENCE:
;	delhdr, name
; INPUTS:
;	name: Image name
; OUTPUTS:
;	No explicit output
; COMMON BLOCKS:
;	IMG_LST: List of image headers
; RESTRICTIONS:
;	Used by IMSYS procedures
; MODIFICATION HISTORY:
;	BBT, July, 1991
;-
on_error, 2
common IMG_LST, ilist
if (n_elements(name) eq 0) then begin
	message, 'Usage: delhdr, name'
endif
if (n_elements(ilist) eq 0) then prolog
if (n_elements(ilist) eq 0) then begin
	message, 'Empty image list'
endif
i = where(ilist.name eq name, count)
if (count gt 1) then begin
	message, 'More than one image: ' + name
endif
if (count eq 0) then begin
	message, 'Image not found: ' + name
endif
ilist(i).header = make_header()
ilist(i).update = 1
return
end

pro newhdr, name, header
;+
; NAME:
;	newhdr
; PURPOSE:
;	Create a new header containing a copy of an old header
; CALLING SEQUENCE:
;	newhdr, name, header
; INPUTS:
;	name:   New image name
;	header: Header from existing image
; OUTPUTS:
;	No explicit output
; COMMON BLOCKS:
;	IMG_LST: List of image headers
; RESTRICTIONS:
;	Used by IMSYS procedures
; MODIFICATION HISTORY:
;	BBT, July, 1991
;-
on_error, 2
common IMG_LST, ilist
if (n_elements(name) eq 0) then begin
	message, 'Usage: newhdr, name, header'
endif
n_keys = n_elements(header)
if (n_keys eq 0) then begin
	message, 'Usage: newhdr, name, header'
endif
image = make_image(1)
i = where(size(image.header(0)) ne size(header(0)), count)
if (count gt 0) then begin
	message, 'Parameter <header> has incorrect type'
endif
if (n_keys gt n_elements(image.header)) then begin
	message, 'Too many keywords'
endif
if (n_elements(ilist) eq 0) then prolog
n = n_elements(ilist)
if (n gt 0) then begin
	i = where(ilist.name eq name, count)
	if (count gt 0) then begin
		message, 'Image already exists: ' + name
	endif
	i = where(ilist.name eq '', count)
	if (count gt 0) then n = i(0) else ilist = [ilist, image]
endif else ilist = [image]
ilist(n).name = name
ilist(n).header = header
ilist(n).update = 1
return
end

pro putkey, name, keyword, value
;+
; NAME:
;	putkey
; PURPOSE:
;	Reset the value of a keyword for a given image 
; CALLING SEQUENCE:
;	putkey, name, keyword, value
; INPUTS:
;	name:    Image name
;	keyword: Keyword name
;	value:   Keyword value
; OUTPUTS:
;	No explicit output
; COMMON BLOCKS:
;	IMG_LST: List of image headers
; RESTRICTIONS:
;	Used by IMSYS procedures
; MODIFICATION HISTORY:
;	BBT, July, 1991
;-
on_error, 2
common IMG_LST, ilist
if (n_elements(name) eq 0) then begin
	message, 'Usage: putkey, name, keyword, value'
endif
if (n_elements(keyword) eq 0) then begin
	message, 'Usage: putkey, name, keyword, value'
endif
if (n_elements(value) eq 0) then begin
	message, 'Usage: putkey, name, keyword, value'
endif
if (n_elements(ilist) eq 0) then prolog
if (n_elements(ilist) eq 0) then begin
	message, 'Empty image list'
endif
i = where(ilist.name eq name, count)
if (count gt 1) then begin
	message, 'More than one image: ' + name
endif
if (count eq 0) then begin
	message, 'Image not found: ' + name
endif
k = where(ilist(i).header.keyword ne '', count)
if (count eq 0) then ilist(i) = rdheader(ilist(i))
k = where(ilist(i).header.keyword eq keyword, count)
if (count gt 1) then begin
	message, 'More than one keyword: ' + keyword
endif
if (count eq 0) then begin
	k = where(ilist(i).header.keyword eq '', count)
	if (count eq 0) then begin
		message, 'All keywords occupied'
	endif
	if (count gt 1) then k = k(0)
	ilist(i).header(k).keyword = keyword
endif
ilist(i).header(k).value = value
if (value eq '') then ilist(i).header(k).keyword = '' 
ilist(i).update = 1
return
end

function getsize, name
;+
; NAME:
;	getsize
; PURPOSE:
;	Get IDL size information for a given image
; CALLING SEQUENCE:
;	<isize> = getsize(name)
; INPUTS:
;	name: Image name
; OUTPUTS:
;	<isize>: Long array containing IDL size information
; COMMON BLOCKS:
;	None
; RESTRICTIONS:
;	Used by IMSYS procedures
; MODIFICATION HISTORY:
;	BBT, July, 1991
;-
on_error, 2
if (n_elements(name) eq 0) then begin
	message, 'Usage: isize = getsize(name)'
endif
str = getkey(name, 'NAXIS')
if (str eq '') then begin
	message, 'Keyword NAXIS not defined: ' + name
endif
naxis = long(str)
if (naxis lt 1) then begin
	message, 'Number of dimensions < 1 : ' + name
endif
ret = lonarr(naxis+3)
ret(0) = naxis
for n = 1, naxis do begin
	str = getkey(name, 'NAXIS'+string(n, format='(i0)'))
	if (str eq '') then begin
		message, 'Keyword NAXISn not defined: ' + name
	endif
	naxis_n = long(str)
	if (naxis_n lt 1) then begin
		message, 'Dimension < 1 : ' + name
	endif
	ret(n) = naxis_n
endfor
str = getkey(name, 'PIXTYPE')
if (str eq '') then begin
	if (getkey(name, 'SIMPLE') eq 'T') then begin
		str = getkey(name, 'BITPIX')
		if (str ne '') then begin
			case fix(str) of
				   8: ImsysType = 1
				  16: ImsysType = 2
				 -32: ImsysType = 3
				  32: ImsysType = 4
				else: ImsysType = 0
			endcase
		endif else ImsysType = 0
	endif else ImsysType = 0
endif else ImsysType = fix(str)
case ImsysType of
	0: DataType = 0
	1: DataType = 1
	2: DataType = 2
	3: DataType = 4
	4: DataType = 3
	5: DataType = 6
	else: DataType = 0
endcase
ret(naxis+1) = DataType
s = ret(1)
for n = 2, naxis do begin
	s = s*ret(n)
endfor
ret(naxis+2) = s
return, ret
end

pro putsize, name, isize
;+
; NAME:
;	putsize
; PURPOSE:
;	Enter image header keywords from IDL size information
; CALLING SEQUENCE:
;	putsize, name, isize
; INPUTS:
;	name:  Image name
;	isize: Long array containing IDL size information for array
; OUTPUTS:
;	No explicit output
; COMMON BLOCKS:
;	None
; RESTRICTIONS:
;	Used by IMSYS procedures
; MODIFICATION HISTORY:
;	BBT, July, 1991
;-
on_error, 2
if (n_elements(name) eq 0) then begin
	message, 'Usage: putsize, name, isize'
endif
if (n_elements(isize) eq 0) then begin
	message, 'Usage: putsize, name, isize'
endif
naxis = isize(0)
if (naxis lt 1) then begin
	message, 'Number of dimensions < 1'
endif
for n = 1, naxis do if (isize(n) lt 1) then begin
	message, 'Dimension < 1'
endif
DataType = isize(naxis+1)
case DataType of
	0: pixtype = 0
	1: pixtype = 1
	2: pixtype = 2
	3: pixtype = 4
	4: pixtype = 3
	5: pixtype = 0
	6: pixtype = 5
	else: pixtype = 0
endcase
if (pixtype eq 0) then begin
	message, 'Illegal pixel type'
endif
case pixtype of
	1: simple = 'T'
	2: simple = 'T'
	3: simple = 'T'
	4: simple = 'T'
	5: simple = 'F'
endcase
case pixtype of
	1: bitpix = '8'
	2: bitpix = '16'
	3: bitpix = '-32'
	4: bitpix = '32'
	5: bitpix = '-64'
endcase
case pixtype of
	1: datamin = '0'
	2: datamin = '-32767'
	3: datamin = '-9E+37'
	4: datamin = '-2147483647'
	5: datamin = '-9E+37'
endcase
case pixtype of
	1: datamax = '255'
	2: datamax = '32767'
	3: datamax = '9E+37'
	4: datamax = '2147483647'
	5: datamax = '9E+37'
endcase
case pixtype of
	1: blank = ''
	2: blank = '-32768'
	3: blank = '1E+38'
	4: blank = '-2147483648'
	5: blank = '1E+38'
endcase
putkey, name, 'SIMPLE', simple
putkey, name, 'BITPIX', bitpix
putkey, name, 'NAXIS', string(naxis, format='(i0)')
for n = 1, naxis do begin
	naxis_n = isize(n)
	keyword = 'NAXIS'+string(n, format='(i0)')
	value = string(naxis_n, format='(i0)')
	putkey, name, keyword, value
endfor
putkey, name, 'PIXTYPE', string(pixtype, format='(i0)')
putkey, name, 'DATAMIN', datamin
putkey, name, 'DATAMAX', datamax
putkey, name, 'BLANK', blank
return
end

pro getim, name, array
;+
; NAME:
;	getim
; PURPOSE:
;	Get image array for a given image
; CALLING SEQUENCE:
;	getim, name, array
; INPUTS:
;	name: Image name
; OUTPUTS:
;	array: Image array
; COMMON BLOCKS:
;	DIR_ENV: Header and image directories, hdr_dir, img_dir
; RESTRICTIONS:
;	None
; MODIFICATION HISTORY:
;	BBT, December, 1992
;-
on_error, 2
on_ioerror, io_err
common DIR_ENV, hdr_dir, img_dir
if (n_elements(name) eq 0) then begin
	message, 'Usage: getim, name, array'
endif
isize = getsize(name)
if (isize(0) eq 0) then begin
	message, 'Not a proper image: ' + name
endif
file = getkey(name, 'FILE')
if (file eq '') then begin
	message, 'Image file not found: ' + name
endif
cd, img_dir, current=old_dir
openu, unit, file, /get_lun
cd, old_dir
a = assoc(unit, make_array(size=isize, /nozero))
array = a(0)
goto, finish
io_err:
on_ioerror, null
message, !err_string, /noname, /ioerror
finish:
free_lun, unit
return
end

pro putim, name, array
;+
; NAME:
;	putim
; PURPOSE:
;	Write IDL image array onto a given image
; CALLING SEQUENCE:
;	putim, name, array
; INPUTS:
;	name:  Image name
;	array: Image array
; OUTPUTS:
;	No explicit output
; COMMON BLOCKS:
;	DIR_ENV: Header and image directories, hdr_dir, img_dir
; RESTRICTIONS:
;	None
; MODIFICATION HISTORY:
;	BBT, December, 1992
;-
on_error, 2
on_ioerror, io_err
common DIR_ENV, hdr_dir, img_dir
if (n_elements(name) eq 0) then begin
	message, 'Usage: putim, name, array'
endif
if (n_elements(array) eq 0) then begin
	message, 'Usage: putim, name, array'
endif
asize = size(array)
if (asize(0) eq 0) then begin
	message, 'Array is scalar or undefined'
endif
isize = getsize(name)
i = where(asize ne isize, count)
if (count gt 0) then begin
	message, 'Array and image are not of the same type'
endif
file = getkey(name, 'FILE')
if (file eq '') then begin
	message, 'Image file not found: ' + name
endif
get_lun, unit
cd, img_dir, current=old_dir
openu, unit, file
cd, old_dir
a = assoc(unit, make_array(size=isize, /nozero))
a(0) = array
putkey, name, 'HISTORY', 'IDL;' + getkey(name, 'HISTORY')
epilog
goto, finish
io_err:
on_ioerror, null
message, !err_string, /noname, /ioerror
finish:
free_lun, unit
return
end

pro newim, name, array, old_name
;+
; NAME:
;	newim
; PURPOSE:
;	Create a new image from a data array and an old image header
; CALLING SEQUENCE:
;	newim, name, array, old_name
; INPUTS:
;	name:     New image name
;	array:    Data array
;	old_name: Old image name
; OUTPUTS:
;	No explicit output
; COMMON BLOCKS:
;	DIR_ENV: Header and image directories, hdr_dir, img_dir
; RESTRICTIONS:
;	None
; MODIFICATION HISTORY:
;	BBT, December, 1992
;-
on_error, 2
on_ioerror, io_err
common DIR_ENV, hdr_dir, img_dir
if (n_elements(name) eq 0) then begin
	message, 'Usage: newim, name, array, old_name'
endif
if (n_elements(array) eq 0) then begin
	message, 'Usage: newim, name, array, old_name'
endif
isize = size(array)
if (isize(0) eq 0) then begin
	message, 'Array is scalar or undefined'
endif
if (n_elements(old_name) eq 0) then begin
	newhdr, name, make_header()
endif else newhdr, name, gethdr(old_name)
putsize, name, isize
file = name + '.IDL'
putkey, name, 'FILE', file
get_lun, unit
cd, img_dir, current=old_dir
openw, unit, file
cd, old_dir
a = assoc(unit, make_array(size=isize, /nozero))
a(0) = array
putkey, name, 'HISTORY', name + ';IDL;' + getkey(name, 'HISTORY')
epilog
goto, finish
io_err:
on_ioerror, null
message, !err_string, /noname, /ioerror
finish:
free_lun, unit
return
end

pro irm, name
;+
; NAME:
;	irm
; PURPOSE:
;	Delete (remove) an image
; CALLING SEQUENCE:
;	irm, name
; INPUTS:
;	name: Image name
; OUTPUTS:
;	No explicit output
; COMMON BLOCKS:
;	DIR_ENV: Header and image directories, hdr_dir, img_dir
; RESTRICTIONS:
;	None
; MODIFICATION HISTORY:
;	BBT, December, 1992
;-
on_error, 2
on_ioerror, io_err
common DIR_ENV, hdr_dir, img_dir
if (n_elements(name) eq 0) then begin
	message, 'Usage: irm, name'
endif
file = getkey(name, 'FILE')
if (file eq '') then begin
	message, 'Image file not found: ' + name
endif
cd, img_dir, current=old_dir
openw, unit, file, /get_lun, /delete
close, unit
cd, old_dir
delhdr, name
epilog
goto, finish
io_err:
on_ioerror, null
message, !err_string, /noname, /ioerror
finish:
free_lun, unit
return
end

