
/**************************************************************************/
/*                                                                        */
/*   This program is free software; you can redistribute it and/or modify */
/*   it under the terms of the GNU General Public License as published by */
/*   the Free Software Foundation; either version 2 of the License, or    */
/*   (at your option) any later version.                                  */
/*                                                                        */
/*   This program is distributed in the hope that it will be useful,      */
/*   but WITHOUT ANY WARRANTY; without even the implied warranty of       */
/*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        */
/*   GNU General Public License for more details.                         */
/*                                                                        */
/*   You should have received a copy of the GNU General Public License    */
/*   along with this program; if not, write to the                        */
/*       Free Software Foundation, Inc.,                                  */
/*       59 Temple Place, Suite 330,                                      */
/*       Boston, MA  02111-1307                                           */
/*       USA                                                              */
/*                                                                        */
/**************************************************************************/
/*
Module Name:
	daa - dynamic array allocator

Description:
	This dynamic array allocator is designed to be efficient,
	i.e., it uses only one malloc() for the entire array, and
	flexible. It can allocate arrays of up to 10 dimensions of
	any type that will return a size with the sizeof "C" operator.
	It is also very efficient from the point of view of array access.
	The single malloc() ensures locality of reference that eliminates
	excess paging encountered with dynamic array allocation schemes
	that use multiple malloc()'s. Unless the array is larger than a
	full page it will be contained entirely on one page. It can
	allocate arrays of structure, enum or union type. A corresponding
	free(ptr) of the allocated area must normally be done. The sixth
	parameter returns a pointer to do a free on. Do NOT free on the
	function return pointer since arrays of non-zero subscripted first
	dimensions will not point to the allocated space, but to some
	offset based on the subscript offset of the first dimension.
	The array is initialized to the value pointed to by the last
	parameter, or not initialized if NULL. The last parameter pointer
	should point to something with a size the same as the size of the
	type in the sizeof first argument. Arrays may be allocated to have
	one or more dimensions with non-zero integer (+ or -) start
	subscripts. Thus, arrays may be one based or zero based or any
	based for that matter.

Parameters:
	size        - size of the basic array data object, this will
	              usually be passed from the sizeof() operator
	              in the call

	no_dim      - number of array dimensions

	dimensions  - a single dimensional array of the dimensions of
	              the array to be allocated

	start       - a single dimensional array of integer start subscripts
	              for each corresponding dimension of the dimensions
	              array, may be negative

	err_code    - pointer to returned error code value

	free_ptr    - pointer to the base pointer of the allocated array
	              space for the caller to free

	init_ptr    - initialization pointer parameter (NULL if no
	              initialization is desired)

Return:
	A pointer to char that points to the start of the dynamically
	allocated array area. This will normally need to be cast to the
	type of the subsequent array references. Routine failure will
	return NULL.

Examples:
		char *fptr;
	allocate 3 dimensional 3x3x3 zeroed int array, all zero based
		int ***array;
		int init = 0;
		int d[3] = {3, 3, 3};
		int st[3] = {0, 0, 0};
		array = (int ***)
			daa(sizeof(int), 3, d, st, &err_code, &fptr, &init);
		array[0][2][1] = 1;
		free(fptr);

	allocate 2 dimensional 2x3 double array initialized to
	1.0 and one based
		double **array;
		double init = 1.0;
		int d[2] = {2, 3};
		int st[2] = {1, 1};
		array = (double **)
			daa(sizeof(double), 2, d, st, &err_code, &fptr, &init);
		array[1][2] = 1.5;
		free(fptr);

	allocate 3 dimensional 4x3x2 array of structures initialized
	to the s_init structure and the first index zero based with
	the second index five based and the third index -2 based
		struct s {
			int i;
			double d;
		};
		struct s s_init = {5, 2.5};
		struct s ***array;
		int d[3] = {4, 3, 2};
		int st[3] = {0, 5, -2};
		array = (struct s ***)
		daa(sizeof(struct s), 3, d, st, &err_code, &fptr, &s_init);
		array[1][5][-2].i = 1;
		array[3][6][-2].d = 1.5;
		free(fptr);

Errors:
	Error free operation returns 0 in *err_code. Any of the following
	errors will result in a NULL pointer return and *err_code set to
	the error number:
		 1. sizeof(char) != 1
		 2. sizeof(short *) != sizeof(char *)
		 3. sizeof(int *) != sizeof(char *)
		 4. sizeof(long *) != sizeof(char *)
		 5. sizeof(float *) != sizeof(char *)
		 6. sizeof(double *) != sizeof(char *)
		 7. size < 1
		 8. number of dimensions < 1 or > 10
		 9. no dimension may be zero
		10. memory must be successfully allocated

	Failure to index the subscripts in the manner established by the
	starting index array will of course result in run time errors.
	This is not an array allocation error but an array usage error,
	similar to any array access on a static "C" zero based array
	outside the normal 0...n-1 bounds.
*/
/*--------------------------------------------------------------------*/
#include "host.h"	/* defines MSC, HC1, HC2, WC, BSD or SYSV */
#include <stdio.h>
#if	defined(BSD)	/* BSD-UNIX */
#include <sys/types.h>	/* size_t defined */
#endif
/*--------------------------------------------------------------------*/
#if	defined(ANSI)	/* ANSI-C */
#include <stdlib.h>
#else	/* not ANSI-C */
#if	defined(BSD)	/* BSD-UNIX */
extern char *calloc();
extern char *malloc();
extern char *realloc();
#endif
#if	defined(SYSV)	/* UNIX System V */
extern void *calloc();
extern void *malloc();
extern void *realloc();
#endif
#endif
/*--------------------------------------------------------------------*/
#include "nonansi.h"	/* const and void redefined if not ANSI-C */
#include "daa.h"	/* function declarations for daa.c */
/*--------------------------------------------------------------------*/
#include "vmalloc.h"	/* function declarations for vmalloc.c */
#include "chkheap.h"	/* redefine malloc functions */
/*--------------------------------------------------------------------*/
#define	MAX_DIM	10	/* max no. of array dimensions */
/*--------------------------------------------------------------------*/
/* these variables are used in the recursive pointer setup routines */
static size_t data_size = 0;	/* data unit size */
static size_t num_dim = 0;	/* no. of dimensions */
static size_t dim[MAX_DIM];	/* dimensions of array */
	/* product of dimensions from 0 to index, dim[0]*..dim[index] */
static size_t dp[MAX_DIM];
static int st[MAX_DIM];	/* start index of array dimensions */
static size_t ptr_size = sizeof(char *);	/* size of pointer */
static char *base = (char *)0;	/* points to base of allocated array */
/*--------------------------------------------------------------------*/
#if	defined(ANSI)	/* ANSI-C */
static size_t off(size_t level)
#else	/* old style */
static size_t off(level)
size_t level;
#endif
{
	if (level == 0)
		return 0;
	else
		return dp[level-1] + off(level-1);
}
/*--------------------------------------------------------------------*/
#if	defined(ANSI)	/* ANSI-C */
static size_t doff(size_t level, size_t dim_ind[], size_t dim_prod)
#else	/* old style */
static size_t doff(level, dim_ind, dim_prod)
size_t level;
size_t dim_ind[];
size_t dim_prod;
#endif
{
	if (level == 0)
		return 0;
	else
		return doff(level-1, dim_ind, dim_prod) +
			(dim_prod/dp[level-1])*dim_ind[level-1];
}
/*--------------------------------------------------------------------*/
#if	defined(ANSI)	/* ANSI-C */
static char *al(size_t level, size_t dim_ind[])
#else	/* old style */
static char *al(level, dim_ind)
size_t level;
size_t dim_ind[];
#endif
{
	char **ptrs;
	size_t i;

	if (level < num_dim - 1) {
		ptrs = (char **)(base + off(level)*ptr_size +
			((level == 0)? (size_t)0:
			doff(level, dim_ind, dp[level])*ptr_size));
		for (i = 0; i < dim[level]; i++) {
			dim_ind[level] = i;
			ptrs[i] = al(level+1, dim_ind) - st[level+1]*
				((level+1 == num_dim-1)? data_size: ptr_size);
		}
		if (level == 0)
			ptrs -= st[0];
	} else
		ptrs = (char **)(base + off(level)*ptr_size +
			doff(level, dim_ind, dp[level])*data_size -
			((level == 0)? (st[0]*data_size): (size_t)0));

	return (char *)ptrs;
}
/*--------------------------------------------------------------------*/
#if	defined(ANSI)	/* ANSI-C */
void *daa(
	unsigned size,
	unsigned no_dim,
	int dimensions[],
	int start[],
	unsigned *err_code,
	char **free_ptr,
	void *init_ptr)
#else	/* old style */
void *daa(size, no_dim, dimensions, start, err_code, free_ptr, init_ptr)
unsigned size;
unsigned no_dim;
int dimensions[];
int start[];
unsigned *err_code;
char **free_ptr;
void *init_ptr;
#endif
{
	size_t data_off;	/* offset in pointer words of first data byte */
	size_t dim_ind[MAX_DIM];	/* array of current dimension indices */
	size_t i, j;
	char *p_data;	/* pointer to array data */
	char *p;	/* tmp pointer */

	*err_code = 0;

	if (sizeof(char) != 1) {
		*err_code = 1;
		return NULL;
	}

	if (sizeof(short *) != sizeof(char *)) {
		*err_code = 2;
		return NULL;
	}

	if (sizeof(int *) != sizeof(char *)) {
		*err_code = 3;
		return NULL;
	}

	if (sizeof(long *) != sizeof(char *)) {
		*err_code = 4;
		return NULL;
	}

	if (sizeof(float *) != sizeof(char *)) {
		*err_code = 5;
		return NULL;
	}

	if (sizeof(double *) != sizeof(char *)) {
		*err_code = 6;
		return NULL;
	}

	if (size < 1) {
		*err_code = 7;
		return NULL;
	}

	if (no_dim < 1 || no_dim > MAX_DIM) {
		*err_code = 8;
		return NULL;
	}

	num_dim = no_dim;

	/* set dim_ind to all zeros, the multiple invocations of the recursive
	   array allocation function al() will pass changed by 1 dim_ind arrays
	   to each invocation of al(). set dim[] and dp[] from dimensions[]
	   input array, and st[] from start[] input array
	*/
	dp[0] = dimensions[0];
	for (i = 0; i < num_dim; i++) {
		dim_ind[i] = 0;
		if (dimensions[i] == 0) {
			*err_code = 9;
			return NULL;
		}
		dim[i] = dimensions[i];
		if (i > 0)
			dp[i] = dp[i-1]*dimensions[i];
		st[i] = start[i];
	}
	data_size = size;

	/* allocate enough memory for the data plus all the array pointers */
	if ((base = (char *)malloc((data_off = off(num_dim-1))*ptr_size +
			dp[num_dim-1]*data_size)) == NULL) {
		*err_code = 10;
		return NULL;
	}

	/* return allocated pointer that caller should use to free space */
	*free_ptr = base;

	/* if init_ptr is NULL skip initialization */
	if (init_ptr != NULL) {
		/* set p_data to point to the start of the data area */
		p_data = base + data_off*ptr_size;
		for (i = 0; i < dp[num_dim-1]; i++) {
			p = (char *)init_ptr;
			for (j = 0; j < data_size; j++)
				*p_data++ = *p++;
		}
	}
	/* do array setup i.e. all the pointer stuff */
	return (void *)al((size_t)0, dim_ind);
}
